#pragma once

#include "CVector3.h"
#include "CVertex.h"
#include "CBuffers.h"

#include <vector>
#include <map>

namespace Combustion
{
	/*
		Less than comparison for two vertices
		(Used in the map to remove duplicates)
	*/
	struct VertexMapComp
	{
		bool operator() (const CVertex& lhs, const CVertex& rhs) const
		{
			if (lhs.pos.Z < rhs.pos.Z) return true; if (lhs.pos.Z > rhs.pos.Z) return false;
			if (lhs.pos.Y < rhs.pos.Y) return true; if (lhs.pos.Y > rhs.pos.Y) return false;
			return lhs.pos.X < rhs.pos.X;
		}
	};

	class Block
	{
	public:
		Block();
		~Block();

		void WipeArray();

		void InitBlock(Terrain* terrain, CVector3 terrainPosition, float block_size_xz, float block_size_y, int dimension, CVector3& position);

		int GetDimensions();
		CVector3 GetPosition();
		float GetDataAt(int col, int row, int depth);
		void SetDataAt(float value, int col, int row, int depth);

		CVector3 VertexInterp(float isoLevel, int index1, int index2, int col, int row, int depth);

		int GetTableIndexAt(int col, int row, int depth);
		float GetIsoValueAtIndex(int index, int col, int row, int depth);
		CVector3 GetPositionAt(int index, int col, int row, int depth);

		void GetTrianglesAt(int col, int row, int depth, std::map<CVertex, int, VertexMapComp>& vertMap,
								std::vector<CVertex>& verticesList, std::vector<unsigned int>& indices, int& idxCount);

		void InitBuffers();
	
		void SetInitialized(bool newValue) { isInitialized = newValue; }
	private:
		Terrain* parent;
		bool isInitialized;
		bool isEmpty;
		int dimensions;
		CVector3 pos;

		//Coordinate dimensions:
		//         +---+
		//        /   /|      
		//(0,0,0)+---+ |      
		//       |   | +(n, n, n) 
		//       |___|/ 
		float* data;
		VBO* vbo;
		IBO* ibo;

		float voxel_size_xz;		//size of an individual voxel
		float voxel_size_y;			//size of an individual voxel
		CVector3 terrainPos;

		void GetBlockLocation(int& col, int& row, int& depth, int index);
		CVector3 VertexInterpolate(float isoValue, CVector3& p1, CVector3& p2, float pVal1, float pVal2);
	};
}